package cs636.pizza.service;

import cs636.pizza.dao.DbDAO;
import cs636.pizza.dao.MenuDAO;
import cs636.pizza.dao.AdminDAO;
import cs636.pizza.dao.PizzaOrderDAO;
import cs636.pizza.dao.PizzaOrderRepository;
import cs636.pizza.dao.ToppingRepository;
import cs636.pizza.domain.MenuTopping;
import cs636.pizza.domain.PizzaOrder;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * This class captures the business logic for admin-related interactions, the
 * actions that ordinary users (students) don't need.
 * 
 * Only one instance of this class is instantiated, i.e., its object is a
 * singleton object, and this singleton receives references to the singleton DAO
 * objects at its own creation time.
 */
// With Spring Data @Transactional, Spring inserts before and after handlers, so database abort
// causes an exception from the after handler, which is not *inside* this method,
// so you can't catch and rethrow as ServiceException.
// But it does handle it responsibly without effort on our parts

@Service
public class AdminService {

	@Autowired
	private ToppingRepository topRep;
	@Autowired
	private PizzaOrderRepository pizzaOrderRep;
	@Autowired
	private DbDAO dbDAO;
	@Autowired
	private AdminDAO adminDAO;
	@Autowired
	private MenuDAO menuDAO;
	@Autowired
	private PizzaOrderDAO pizzaOrderDAO;

	public AdminService() {
	}

	@Transactional
	public void initializeDb() throws ServiceException {
		dbDAO.initializeDb();
	}

	@Transactional
	public void addTopping(String name) throws ServiceException {
		menuDAO.createMenuTopping(name);
	}
	
	// Spring Data Repository: defined by one interface, supplies lots of methods...

	@Transactional
	public void addTopping1(MenuTopping top) {
		System.out.println("addTopping1, see top of " + top.getToppingName());
		topRep.save(top);
	}

	@Transactional
	public void removeTopping1(MenuTopping top) {
		System.out.println("removeTopping1, see top of " + top.getToppingName());
		topRep.delete(top);
	}
	
	@Transactional
	public MenuTopping findByName(String name)  {
		System.out.println("findByToppingName, name = " + name);
		return topRep.findByToppingName(name);
	}
	
	@Transactional
	public void removeTopping(String topping) {
		menuDAO.deleteMenuTopping(topping);
	}

	@Transactional
	public void addPizzaSize(String name) throws ServiceException {
		menuDAO.createMenuSize(name);
	}

	@Transactional
	public void removePizzaSize(String size) throws ServiceException {
		menuDAO.deleteMenuSize(size);
	}

	@Transactional
	public void markNextOrderReady() throws ServiceException {
		PizzaOrder order = null;
		order = pizzaOrderDAO.findFirstOrder(PizzaOrder.PREPARING);
		if (order == null) {
			throw new ServiceException("No PREPARING orders exist!");
		}
		order.makeReady(); // this change is tracked by JPA
	}

	@Transactional
	public void markNextOrderReady1() throws ServiceException {
		PizzaOrder order = null;
		order = (pizzaOrderRep.findByStatusOrderById(PizzaOrder.PREPARING)).iterator().next();
		if (order == null) {
			throw new ServiceException("No PREPARING orders exist!");
		}
		order.makeReady(); // this change is tracked by JPA
	}
	@Transactional
	public int getCurrentDay() {
		int day;
		day = adminDAO.findCurrentDay();
		return day;
	}

	@Transactional
	public void advanceDay() {
		List<PizzaOrder> pizzaOrders = getTodaysOrders();
		// day is done, so mark today's pizzas as "finished"
		for (PizzaOrder order : pizzaOrders) {
			order.finish();
		}
		adminDAO.advanceDay();
	}

	// helper method to advanceDay
	// executes inside the current transaction
	private List<PizzaOrder> getTodaysOrders() {
		int today = adminDAO.findCurrentDay();
		List<PizzaOrder> orders = pizzaOrderDAO.findOrdersByDays(today, today);
		return orders;
	}
	
	@Transactional
	public List<PizzaOrderData> getOrdersByDay(int day) {
		List<PizzaOrder> orders = pizzaOrderDAO.findOrdersByDays(day, day);
		List<PizzaOrderData> orders1 = new ArrayList<PizzaOrderData>();
		for (PizzaOrder o : orders) {
			orders1.add(new PizzaOrderData(o));
		}
		return orders1;
	}
	@Transactional
	public List<PizzaOrderData> getTodaysOrdersByStatus(int status) {

		List<PizzaOrder> orders = getTodaysOrders();
		List<PizzaOrderData> orders1 = new ArrayList<PizzaOrderData>();
		for (PizzaOrder o : orders) {
			if (o.getStatus() == status)
				orders1.add(new PizzaOrderData(o));
		}
		return orders1;
	}

}
